"""
LTI consumer plugin models
"""
from urllib.parse import unquote

from django.conf import settings
from django.db import models
from django.utils.functional import lazy
from django.utils.translation import gettext_lazy as _

import exrex
from cms.models.pluginmodel import CMSPlugin
from cms.utils import get_current_site
from oauthlib import oauth1

from .defaults import INSTRUCTOR, PREDEFINED_LTI_PROVIDERS, STUDENT


class LTIConsumer(CMSPlugin):
    """
    LTI consumer plugin model.
    """

    url = models.URLField(
        verbose_name=_("LTI url"),
        help_text=_(
            "For a predefined provider, leave this field empty for uploading new content."
        ),
        null=False,
        blank=True,
    )
    lti_provider_id = models.CharField(
        verbose_name=_("Predefined LTI provider"),
        max_length=50,
        null=True,
        blank=True,
        choices=lazy(lambda: PREDEFINED_LTI_PROVIDERS, tuple)(),
        help_text=_("Please choose a predefined provider or fill fields below."),
    )
    oauth_consumer_key = models.CharField(max_length=50, null=True, blank=True)
    shared_secret = models.CharField(max_length=50, null=True, blank=True)

    # pylint: disable=signature-differs
    def save(self, *args, **kwargs):
        """
        Initialize attributes :
        - context_id from current site domain
        - url if predefined provider is used
        """
        if not self.id:
            if self.lti_provider and not self.url:
                if self.lti_provider.get("is_base_url_regex"):
                    self.url = "{:s}".format(
                        exrex.getone(self.lti_provider.get("base_url", "")),
                    )
                else:
                    self.url = self.lti_provider.get("base_url", "")
        super().save(*args, **kwargs)

    @property
    def lti_provider(self):
        """
        Returns current predefined LTI provider settings
        """
        return getattr(settings, "RICHIE_LTI_PROVIDERS", {}).get(
            self.lti_provider_id, {}
        )

    @property
    def automatic_resizing(self):
        """
        Returns current predefined LTI provider automatic_resizing setting
        """
        return self.lti_provider.get("automatic_resizing", False)

    @property
    def inline_ratio(self):
        """
        Returns current predefined LTI provider inline_ratio setting
        """
        return self.lti_provider.get("inline_ratio", 0)

    @property
    def inline_ratio_css_padding_bottom(self):
        """
        Returns current predefined LTI provider inline_ratio css style
        """
        return f"padding-bottom: {self.inline_ratio * 100}%"

    def auth_parameters(self, edit=False):
        """
        Builds required parameters for LTI authentication
        """
        role = INSTRUCTOR if edit else STUDENT
        site = get_current_site()
        return {
            "lti_message_type": self.lti_provider.get(
                "display_name", self.lti_provider_id
            ),
            "lti_version": "LTI-1p0",
            "resource_link_id": str(self.id),
            "context_id": site.domain,
            "user_id": "richie",
            "lis_person_contact_email_primary": "",
            "roles": role,
        }

    def authorize(self):
        """
        Returns headers from LTI authentication
        """
        client = oauth1.Client(
            client_key=self.lti_provider.get("oauth_consumer_key"),
            client_secret=self.lti_provider.get("shared_secret"),
        )

        _uri, headers, _body = client.sign(
            self.url,
            http_method="POST",
            body=self.auth_parameters(),
            headers={"Content-Type": "application/x-www-form-urlencoded"},
        )
        return headers

    def build_content_parameters(self, auth_headers, edit=False):
        """
        Builds required parameters for LTI content consumption
        """
        oauth_dict = dict(
            param.strip().replace('"', "").split("=")
            for param in auth_headers["Authorization"].split(",")
        )
        signature = oauth_dict["oauth_signature"]
        oauth_dict["oauth_signature"] = unquote(signature)
        oauth_dict["oauth_nonce"] = oauth_dict.pop("OAuth oauth_nonce")

        oauth_dict.update(self.auth_parameters(edit=edit))
        return oauth_dict

    def content_parameters(self, edit=False):
        """
        Convenient wrapper to authorize and return required parameters
        for LTI content consumption
        """
        return self.build_content_parameters(self.authorize(), edit=edit)

    def __str__(self):
        return self.url



"""
LTIConsumer CMS plugin factories
"""
import factory

from .models import LTIConsumer


class LTIConsumerFactory(factory.django.DjangoModelFactory):
    """
    Factory to create random instances of LTIConsumer for testing.
    """

    class Meta:
        model = LTIConsumer

    url = factory.Faker("url")
    lti_provider_id = "lti_provider_test"
    oauth_consumer_key = None
    shared_secret = None



"""LTI Consumer plugin forms."""
from django import forms
from django.utils.translation import gettext_lazy as _

from .models import LTIConsumer


class LTIConsumerForm(forms.ModelForm):
    """Plugin form used to add a LTI provided content"""

    class Meta:
        """Form meta attributes"""

        model = LTIConsumer
        fields = [
            "lti_provider_id",
            "url",
            "oauth_consumer_key",
            "shared_secret",
        ]

    def clean(self):
        """
        This form can be filled in two ways:
        - use a predefined LTI provider by filling lti_provider_id field
        - use a custom LTI provider by filling others fields

        Here we're adding related errors
        """
        if not self.cleaned_data.get("lti_provider_id"):
            missing_url = not self.cleaned_data.get("url")
            missing_oauth_consumer_key = not self.cleaned_data.get("oauth_consumer_key")
            missing_shared_secret = not self.cleaned_data.get("shared_secret")

            if missing_url and missing_oauth_consumer_key and missing_shared_secret:
                self.add_error(
                    "lti_provider_id",
                    _("Please choose a predefined provider, or fill fields below"),
                )
                error_message = _(
                    "Please choose a predefined provider above, or fill this field"
                )
                self.add_error("url", error_message)
                self.add_error("oauth_consumer_key", error_message)
                self.add_error("shared_secret", error_message)
            else:
                error_message = _("Please fill this field")
                if missing_url:
                    self.add_error("url", error_message)
                if missing_oauth_consumer_key:
                    self.add_error("oauth_consumer_key", error_message)
                if missing_shared_secret:
                    self.add_error("shared_secret", error_message)
        return super().clean()



"""
LTI consumer CMS plugin
"""
import json

from django.utils.translation import gettext_lazy as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from richie.apps.core.defaults import PLUGINS_GROUP

from .forms import LTIConsumerForm
from .models import LTIConsumer


@plugin_pool.register_plugin
class LTIConsumerPlugin(CMSPluginBase):
    """
    A plugin to consume LTI content.
    """

    allow_children = False
    cache = True
    disable_child_plugins = True
    form = LTIConsumerForm
    model = LTIConsumer
    module = PLUGINS_GROUP
    name = _("LTI consumer")
    render_template = "richie/lti_consumer/lti_consumer.html"

    class Media:
        """
        Simple UX improvement that hides useless fields if a predefined LTI provider is used
        """

        js = ("lti_consumer/js/change_form.js",)

    def render(self, context, instance, placeholder):
        """
        Build plugin context passed to its template to perform rendering
        and pass edit mode
        """
        edit = (
            "request" in context
            and context["request"].toolbar
            and context["request"].toolbar.edit_mode_active
        )

        context = super().render(context, instance, placeholder)
        context["widget_props"] = json.dumps(
            self.get_lti_consumer_widget_props(instance, edit=edit)
        )
        return context

    @staticmethod
    def get_lti_consumer_widget_props(instance, edit=False):
        """
        Return all LTI consumer properties required by LTIConsumer React widget
        """
        return {
            "url": instance.url,
            "content_parameters": instance.content_parameters(edit=edit),
            "automatic_resizing": instance.automatic_resizing,
        }
